<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

use FiloBlu\MappConnector\Model\Config;
use FiloBlu\MappConnector\Model\To\Mapp\User;
use Magento\Customer\Api\CustomerRepositoryInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DataObject;
use Magento\Framework\Event\Manager;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Subscriber
 * @package FiloBlu\MappConnector\Model\From\Entity
 */
class Subscriber extends AbstractEntity implements EntityInterface
{

    /**
     * Entity
     *
     * @var string
     */
    protected $_entity = 'subscriber';

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel = \Magento\Newsletter\Model\Subscriber::class;

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary = 'subscriber_id';

    /**
     * Magento Entity Model Related Id
     *
     * @var string
     */
    protected $_entityModelRelatedPrimary = 'customer_id';

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource = User::class;

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary = ['primaryDB' => 'subscriber_email', 'primaryMapp' => 'email'];

    /**
     * User Type
     *
     * @var string
     */
    protected $_userType = 'guest';

    /**
     * @var string
     */
    protected $_xmlConfigPath = 'mappconnector/subscriber_mapping/subscriber';

    /**
     * @var Customer
     */
    protected $_customer;

    /**
     * @var Config
     */
    protected $_config;

    /**
     * @var StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Framework\Module\Manager
     */
    protected $_moduleManager;


    /**
     * @var Manager
     */
    protected $_eventManager;
    /**
     * @var CustomerRepositoryInterface
     */
    private $customerRepository;

    /**
     * Subscriber constructor.
     * @param Customer $customer
     * @param Config $config
     * @param StoreManagerInterface $storeManager
     * @param \Magento\Framework\Module\Manager $moduleManager
     * @param ResourceConnection $resourceConnection
     * @param Manager $eventManager
     * @param CustomerRepositoryInterface $customerRepository
     */
    public function __construct(
        Customer $customer,
        Config $config,
        StoreManagerInterface $storeManager,
        \Magento\Framework\Module\Manager $moduleManager,
        ResourceConnection $resourceConnection,
        Manager $eventManager,
        CustomerRepositoryInterface $customerRepository
    ) {
        parent::__construct($resourceConnection);
        $this->_customer = $customer;
        $this->_config = $config;
        $this->_storeManager = $storeManager;
        $this->_moduleManager = $moduleManager;
        $this->_eventManager = $eventManager;
        $this->customerRepository = $customerRepository;
    }

    /**
     * @param $record
     * @return array @inheritDoc
     * @throws \Magento\Framework\Exception\LocalizedException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function getDefaultData($record)
    {
        $result = [];
        $result['store_code'] = $this->_storeManager->getStore($record['store_id'])->getCode();
        $result['action'] = 'create';
        $result['type'] = ($record['customer_id'] == 0) ? 'GUEST' : 'CUSTOMER';
        $result['subscriber_status_normalized'] = ($record['subscriber_status'] == 1) ? "1" : "0";
        $result['customer_created_at'] = null;

        if (!isset($record['customer_id']) || !$record['customer_id']) {
            return $result;
        }

        $customer = $this->customerRepository->getById($record['customer_id']);
        $result['customer_created_at'] = $customer->getCreatedAt();

        return $result;
    }

    /**
     * @param $primary
     * @param $id
     * @return mixed @inheritDoc
     */
    public function getObjectData($primary, $id)
    {
        $connection = $this->resourceConnection->getConnection();
        $tableName = $this->resourceConnection->getTableName('newsletter_subscriber'); //gives table name with prefix

        //Select Data from table
        $sql = "SELECT * FROM {$tableName} WHERE {$primary} = {$id}";
        $result = $connection->fetchAll($sql);

        return $result[0];
    }

    /**
     * @param string $type
     * @param string $last_index
     * @return mixed
     */
    public function getAllData($type = 'full', $last_index = '')
    {
        $connection = $this->resourceConnection->getConnection();
        $tableName = $this->resourceConnection->getTableName('newsletter_subscriber'); //gives table name with prefix
        $tableNameJobs = $this->resourceConnection->getTableName('filoblu_mappconnector_jobs');

        if ($type == 'full') {
            return $connection->fetchAll("SELECT * FROM {$tableName}");
        } else {
            //Incremental
            $sql = "SELECT
                      ce.subscriber_id as subscriber_id
                    FROM
                      " . $tableName . " AS ce
                    LEFT JOIN
                      " . $tableNameJobs . " AS mj
                    ON
                      mj.entity_id = ce.subscriber_id
                    AND
                      mj.entity = 'subscriber'
                    WHERE
                      mj.status IS NULL";

            return $connection->fetchAll($sql);
        }
    }

    /**
     * @param $record
     * @return array
     */
    public function getAdditionalData($record)
    {
        $result = [];

        $result = array_merge(
            $result,
            $this->getRelatedModelData($this->_customer, $record['customer_id'])
        );

        $result = array_merge($result, ['subscriberid' => $record['subscriber_id']]);

        // Dispatch event for newsletter observer
        $subscriber_data = new DataObject(['subscriber_data' => $result]);
        $this->_eventManager->dispatch('mappconnector_subscriber_additional_data', ['result' => $subscriber_data]);
        $result = $subscriber_data->getSubscriberData();

        return $result;
    }
}
